//
// Parago Media GmbH & Co. KG, Jrgen Burle (jbaurle@parago.de)
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//

using System;
using System.Security.Permissions;
using System.Text;
using System.Web;
using System.Web.Caching;
using System.Web.UI;

namespace Parago.Web {

	/// <summary>
	/// The CachePageStatePersister saves and loads view state and control state 
	/// in cache on the server.
	/// </summary>
	[AspNetHostingPermission(
		SecurityAction.Demand,
		Level=AspNetHostingPermissionLevel.Minimal)
	]
	public class CachePageStatePersister : PageStatePersister {

		/// <summary>
		/// Initializes a new instance of the CachePageStatePersister class
		/// given a Page class.
		/// </summary>
		public CachePageStatePersister(Page page)
			: base(page) {
		}

		/// <summary>
		/// Loads the saved view state and control state from cache.
		/// </summary>
		public override void Load() {

			// Get the cache key from the web form data
			string key=Page.Request.Params["__VIEWSTATE_CACHEKEY"] as string;

			// Abort if cache key is not available or valid
			if(String.IsNullOrEmpty(key)||!key.StartsWith("VIEWSTATE_"))
				throw new ApplicationException("Missing vaild __VIEWSTATE_CACHEKEY");

			Pair state=Page.Cache[key] as Pair;

			// Abort if cache object is not of type Pair
			if(state==null)
				throw new ApplicationException("Missing vaild __VIEWSTATE_CACHEKEY");

			// Set view state and control state
			ViewState=state.First;
			ControlState=state.Second;

		}

		/// <summary>
		/// Saves the view state and control state in cache.
		/// </summary>
		public override void Save() {

			// No processing needed if no states available
			if(ViewState==null&&ControlState!=null)
				return;

			StringBuilder key=new StringBuilder();

			// Generate a unique cache key
			key.Append("VIEWSTATE_")
				.Append(Page.Session!=null?Page.Session.SessionID:Guid.NewGuid().ToString())
				.Append("_")
				.Append(Page.Request.RawUrl)
				.Append("_")
				.Append(DateTime.Now.Ticks.ToString());

			// Save view state and control state separately
			Pair state=new Pair(ViewState, ControlState);

			// Add view state and control state to cache
			Page.Cache.Add(
				key.ToString(),
				state,
				null,
				DateTime.Now.AddMinutes(15),
				Cache.NoSlidingExpiration,
				CacheItemPriority.Default,
				null
			);

			// Register hidden field to store cache key in
			Page.ClientScript.RegisterHiddenField("__VIEWSTATE_CACHEKEY", key.ToString());

		}

	}

}
